//+------------------------------------------------------------------+
//|  Last edit: August 8, 2014                        00sd Mu.mq4    |
//|  1)                                                              |
//|  Mu: The Mathematical Average of all 9 TF SMAs.                  |
//|  Also shown are current levels for all 9 TF SMAs                 |
//|  Note that Mu is ALLWAYS found inside this SMA distribution.     |
//|                                                                  |
//|  2)                                                              |
//|  muDev: The Average Deviation of Price in all 9 TF SMAs.         |
//|  muDev is congruent with Bid, since the equation                 |
//|  Mu + muDev = Bid is satisfied in all Time Frames.               |
//|                                                                  |
//|  The outlay of these Market Averages is invariant across TFs.    |
//|  This results in good situational awareness.                     |
//|  On-Tick values only.                                            |
//+------------------------------------------------------------------+
#property copyright " 2014 apipshort"
#property link "http://www.forexfactory.com/showthread.php?t=389386"
#property indicator_chart_window

extern string    Make_Sure = "     Set SMA Period equal to BB system!";
extern string   SMA_Period = "     Default for Simple MA Period = 20";
extern int       smaPeriod = 20;   // user choice

extern string        setMu = "     Mu Settings";
extern int    lenghtMuLine = 100;   // user choice
extern bool      muDevLine = true;  // if false hide mu/bid line
extern bool       useMuDev = true;  // if false calculate the average, slower.
extern int         widthMu = 1;     // 1..
extern bool      showMuRay = false; // if true show a horizontal line
extern color         clrMu = clrDodgerBlue; // color mu line - default only

extern string          set = "     SMA Lines";
extern bool       showSMAs = true;  // true
extern int       widthSMAs = 1;     // 1..
extern color       clrSMAs = clrBlue;  // choose single color for Light Background

extern string   BackGround = "     Dark Screen = 1, Light = 2";
extern int         colorBG = 1;     // 1-2
extern bool    showComment = true;
extern string       Manage = "     Label Settings";
extern string         font = "Arial"; // label font
extern int        fontSize = 10;      // label fontsize
extern int          corner = 1;       // label display

string tf[9] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int   per[9] = {PERIOD_M1,PERIOD_M5,PERIOD_M15,PERIOD_M30,
      PERIOD_H1,PERIOD_H4,PERIOD_D1,PERIOD_W1,PERIOD_MN1};

double tfSMA[9];  // array on 9 TF SMAs
double mu, muDev; // mu values
double muOpen;    // track mu moves since last switch
double xPoint;    // display pips

int line;       // bar position of muDevLine
int jump;       // vertical label spacing
string thisTF;  // current TF
string padding; // offset for right-hand text display
string pref = "o_";  // delete objects
color clrDwn, clrUp; // color muDev label

// colors for 9 TFs, DarkBG only
color col[9] = {clrGray, clrGreen, clrDeepSkyBlue, clrRed, clrLightSeaGreen,
                clrDarkViolet, clrTomato, clrAqua, clrHotPink};

//+------------------------------------------------------------------+

void SetColors(int backGround)
   {
   switch (backGround)
      {
      case 1: clrUp = clrLime; clrDwn = clrCrimson; return;
      case 2: clrUp = clrGreen; clrDwn = clrCrimson; for(int i=0; i<=8 ;i++) col[i] = clrSMAs; return;

      default:     return;
      }
   } // end void SetColors(int backGround)

//+------------------------------------------------------------------+

int init()
   {
   //if found label "00z_SMA" read SMA Period from there
   int getSMA = StrToInteger(ObjectDescription("00z_SMA"));
   if(getSMA > 2) smaPeriod = getSMA;

   //if found label "00z_BG" read BG selection from there
   int getBG = StrToInteger(ObjectDescription("00z_BG"));
   if(getBG > 0) colorBG = getBG;
   if(colorBG < 1 || colorBG > 2) colorBG = 1;
   SetColors(colorBG);

   if(lenghtMuLine < smaPeriod) lenghtMuLine = smaPeriod;
   for (int i=0; i<=8 ;i++) {if(per[i] == Period()) thisTF = tf[i];}

   line = smaPeriod;
   if(smaPeriod >= 100) line = 100; // limit muDevLine drift
   jump = fontSize * 2;

   xPoint = Point;
   if(Digits==5||Digits==3) {xPoint = Point * 10;}
   if(Digits==2) {xPoint = Point * 100;}

   padding = "                           "; // adjust as fit
   //........"..........................."; place holder only

   IndicatorShortName("mu");
   muOpen = 0;

   return(0);
   } // end int init()

//+------------------------------------------------------------------+

int deinit()
   {
   Clear(pref, 0);
   if (showComment) Comment("");

   return(0);
   } // end int deinit()

//+------------------------------------------------------------------+

int start()
   {
   GetMuData();
   ShowMuData();

   return(0);
   } // end start()

//+------------------------------------------------------------------+
void GetMuData()
   {
   double sumTFsma = 0;
   for(int i=0; i<=8 ;i++)
      {
      tfSMA[i] = iMA(NULL, per[i], smaPeriod, 0,MODE_SMA,PRICE_CLOSE, 0);
      sumTFsma += tfSMA[i];
      }
   mu = sumTFsma / 9;
   muDev = Bid - mu; // simplification

   if(!useMuDev)
      {// optional classical averaging, inefficient
      double sumTFdev = 0;
      for(i=0; i<=8 ;i++)
         {
         sumTFdev += Bid - tfSMA[i];
         }
      muDev = sumTFdev / 9;
      } //  end if(!useMuDev)

   if(muOpen == 0) muOpen = mu;
   double muMove = mu - muOpen;

   if(showComment) Comment("SMA ", smaPeriod, "  Mu ", DoubleToStr(mu, Digits), "  muOpen ", DoubleToStr(muOpen, Digits),
                           "  muMove ", GetSign(muMove), DoubleToStr(muMove/xPoint, 2));

   return;
   } // end void GetData()

//+------------------------------------------------------------------+

void ShowMuData()
   {
   // 4 info labels
   int y = jump; // vertical label spacing
   if(muDev < 0) {color clrOpen = clrDwn;} else {clrOpen = clrUp;}

   // tf & sma period info
   string text = StringConcatenate(thisTF, " ", smaPeriod, " sma");
   Label("sma", 15, y, corner, text, fontSize, font, clrMu);

   y += jump; // bid
   text = StringConcatenate("Bid  ", DoubleToStr(Bid, Digits -1));
   Label("bid", 15, y, corner, text, fontSize, font, clrMu);

   y += jump; // mu
   text = StringConcatenate("Mu  ",DoubleToStr(mu, Digits - 1));
   Label("mu", 15, y, corner, text, fontSize, font, clrMu);

   y += jump; // muDev
   text = StringConcatenate("Dev   ", GetSign(muDev), DoubleToStr(muDev / xPoint, 1));
   Label("muDev", 15, y, corner, text, fontSize, font, clrOpen);

   // mu line + 2 markers dropped on init: a mu open-level tag and an SMA Last Bar tag.
   string obj = StringConcatenate("mu ", smaPeriod, " SMA");
   TrendLine(obj, 0, lenghtMuLine, mu, mu, clrMu, 0, widthMu, showMuRay, true);
   TrendLine("mu_tag", 0, 1, muOpen, muOpen, clrMu, 2, widthMu, false, false);

   obj = StringConcatenate("last_bar = ", smaPeriod);
   Arrow(obj, smaPeriod, iLow(NULL, 0, smaPeriod), 115, 2, clrMu, true, 0);

   // an optional vertical line connecting mu and Bid/muDev levels.
   if(muDevLine) TrendLine("mu_bid", line, line, mu, Bid, clrMu, 2, 1, false, false);

   // show all 9 TF SMAs
   for(int i=0; i<=8 ;i++)
      {
      obj = StringConcatenate(tf[i], "_line");
      if(showSMAs)
         { // show all TF SMAs
         TrendLine(obj, 1, 0, tfSMA[i], tfSMA[i], col[i], 0, widthSMAs, false, true);
         obj =  StringConcatenate(tf[i], "_price");
         text = StringConcatenate(padding, DoubleToStr(tfSMA[i], Digits - 1), "  ", tf[i]);
         TextBox(obj, 0, tfSMA[i], text, fontSize, font, col[i]);
         }
         else {Arrow(obj, 0, tfSMA[i], 4, 2, col[i], false, ANCHOR_TOP);
         } // end if(showSMAs)

      } // end for(int i=0
   } // end void ShowMuData()

//+-----------------------------SUPPORT------------------------------+

void TrendLine(string obj, int t1, int t2, double p1, double p2, color clr, int st, int wide, bool ray, bool back)
   {
   obj = StringConcatenate(pref, obj);
   if(!ObjectCreate(obj, OBJ_TREND, 0,0,0,0,0)) ObjectCreate(obj, OBJ_TREND, 0,0,0,0,0);
      if(!ObjectSet(obj, OBJPROP_TIME1, Time[t1])) ObjectSet(obj, OBJPROP_TIME1, Time[t1]);
      if(!ObjectSet(obj, OBJPROP_TIME2, Time[t2])) ObjectSet(obj, OBJPROP_TIME2, Time[t2]);
      if(!ObjectSet(obj, OBJPROP_PRICE1, p1))      ObjectSet(obj, OBJPROP_PRICE1, p1);
      if(!ObjectSet(obj, OBJPROP_PRICE2, p2))      ObjectSet(obj, OBJPROP_PRICE2, p2);
      if(!ObjectSet(obj, OBJPROP_COLOR,  clr))     ObjectSet(obj, OBJPROP_COLOR,  clr);
      if(!ObjectSet(obj, OBJPROP_STYLE,  st))      ObjectSet(obj, OBJPROP_STYLE,  st);
      if(!ObjectSet(obj, OBJPROP_WIDTH,  wide))    ObjectSet(obj, OBJPROP_WIDTH,  wide);
      if(!ObjectSet(obj, OBJPROP_RAY,    ray))     ObjectSet(obj, OBJPROP_RAY,    ray);
      if(!ObjectSet(obj, OBJPROP_BACK,   back))    ObjectSet(obj, OBJPROP_BACK,   back);

   return;
   }// end void TrendLine(

//+------------------------------------------------------------------+

void TextBox(string obj, int t1, double p1, string text, int size, string afont, color clr)
   {
   obj = StringConcatenate(pref, obj);
   if (!ObjectCreate(obj, OBJ_TEXT, 0,0,0,0))    ObjectCreate(obj, OBJ_TEXT, 0,0,0,0);
       if(!ObjectSet(obj, OBJPROP_TIME1, Time[t1])) ObjectSet(obj, OBJPROP_TIME1, Time[t1]);
       if(!ObjectSet(obj, OBJPROP_PRICE1, p1))      ObjectSet(obj, OBJPROP_PRICE1, p1);
   if(!ObjectSetText(obj, text , size, afont, clr)) ObjectSetText(obj, text , size, afont, clr);

   return;
   }// end void TextBox(

//+------------------------------------------------------------------+

void Arrow(string obj, int t1, double p1, int code, int wide, color clr, bool back, int anchr)
   {
   obj = StringConcatenate(pref, obj);
   if(!ObjectCreate(obj, OBJ_ARROW, 0,0,0,0))   ObjectCreate(obj, OBJ_ARROW, 0,0,0,0);
      if(!ObjectSet(obj, OBJPROP_TIME1, Time[t1])) ObjectSet(obj, OBJPROP_TIME1, Time[t1]);
      if(!ObjectSet(obj, OBJPROP_PRICE1, p1))      ObjectSet(obj, OBJPROP_PRICE1, p1);
      if(!ObjectSet(obj, OBJPROP_ARROWCODE, code)) ObjectSet(obj, OBJPROP_ARROWCODE, code);
      if(!ObjectSet(obj, OBJPROP_WIDTH, wide))     ObjectSet(obj, OBJPROP_WIDTH, wide);
      if(!ObjectSet(obj, OBJPROP_COLOR, clr))      ObjectSet(obj, OBJPROP_COLOR, clr);
      if(!ObjectSet(obj, OBJPROP_BACK,  back))     ObjectSet(obj, OBJPROP_BACK,  back);
      if(!ObjectSet(obj, OBJPROP_ANCHOR, anchr))   ObjectSet(obj, OBJPROP_ANCHOR, anchr);

   return;
   }// end void Arrow(

//+------------------------------------------------------------------+

void Label(string obj, int x, int y, int corn, string text, int size, string font_, color clr)
   {
   obj = StringConcatenate(pref, obj);
   if(!ObjectCreate(obj, OBJ_LABEL, 0,0,0))    ObjectCreate(obj, OBJ_LABEL, 0,0,0);
      if(!ObjectSet(obj, OBJPROP_XDISTANCE, x))   ObjectSet(obj, OBJPROP_XDISTANCE, x);
      if(!ObjectSet(obj, OBJPROP_YDISTANCE, y))   ObjectSet(obj, OBJPROP_YDISTANCE, y);
      if(!ObjectSet(obj, OBJPROP_CORNER, corn)) ObjectSet(obj, OBJPROP_CORNER, corn);
   if(!ObjectSetText(obj, text, size, font_, clr)) ObjectSetText(obj, text, size, font_, clr);

   return;
   }// end void Label(

//+------------------------------------------------------------------+

string GetSign(double in)
   {
   if (in <= 0) {string sign = "";} else {sign = "+";}
   return(sign);
   } // end string GetSign(double in)

//+------------------------------------------------------------------+

void Clear(string findMe, int startFrom)
   {
   string name;
   int obj_total = ObjectsTotal();
   for (int i=obj_total-1; i>=0; i--)
      {
       name = ObjectName(i);
       if (StringFind(name, findMe) == startFrom) ObjectDelete(name);
      }
   return;
   } // end void Clear(string findMe, int startFrom)

//+-----------------------------EOP----------------------------------+

/* Developpments:
      Establish an SD for Mu, using TF price deviations.
      Establish the z-score of Bid on Mu.
*/