//+----------------------------------------------------------------------+
//|  Last edit June 8 2014               00sd Normal Bell Curve-demo.mq4 |
//|  Draws a Standard Normal Distribution curve in separate window.      |
//|  The curve consisting of over 600 y values, uses the same number     |
//|  of Bars. Zoom out completely and then one level back or so.         |
//|  This is an SND as discussed on                                      |
//|  http://en.wikipedia.org/wiki/Normal_distribution. MUST: look at     |
//|  http://en.wikipedia.org/wiki/File:Normal_distribution_and_scales.gif|
//+----------------------------------------------------------------------+
#property copyright " 2014 apipshort"
#property link "http://www.forexfactory.com/showthread.php?t=389386"
#property indicator_separate_window
#property indicator_maximum 1
#property indicator_minimum -0.25

#property indicator_buffers 1
#property indicator_color1 DimGray

extern string ScreenColor = "Dark Screen = 1, Light = 2";
extern int        colorBG = 1; // 1
extern int       fontSize = 9; // 11
extern string        font = "Verdana"; //"Arial";

string pref = "bell_";
int    win;
color  clrSig;
int  base = 400;
bool bellDone, gridDone;

double bell[]; // y values for indie buffer

// below: all critical values
double critP[12]  = {0.001, 0.005, 0.010, 0.025, 0.050, 0.100,
                     0.900, 0.950, 0.975, 0.990, 0.995, 0.999};

double critZp[12] = {-3.090, -2.576, -2.326, -1.960, -1.645, -1.282,
                      1.282, 1.645, 1.960, 2.326, 2.576, 3.090};

double percP[7]  = {0.200, 0.300, 0.400, 0.500, 0.600, 0.700, 0.800};
double percZp[7] = {-0.842, -0.524, -0.253, 0.000, 0.253, 0.524, 0.842};

double iqR[4] = {-2.698, -0.6745, 0.6745, 2.698};

//+------------------------------------------------------------------+

void SetColors(int backGround)
   {
   switch (backGround)
      {
      case 1: clrSig = Silver; return;
      case 2: clrSig = Black; return;

      default:     return;
      }
   } // end void SetColors(int backGround)

//+------------------------------------------------------------------+

int init()
   {
   if(colorBG < 1 || colorBG > 2) colorBG = 1;
   SetColors(colorBG);

   IndicatorShortName("Bell_Curve_demo");
   SetIndexBuffer(0,bell);
    SetIndexStyle(0,DRAW_SECTION,STYLE_DOT);
    SetIndexLabel(0,"y");

   return(0);
   } // end int init()

//+------------------------------------------------------------------+

int deinit()
   {
   return(0);
   } // end int deinit()

//+------------------------------------------------------------------+

int start()
   {
   if(win < 1)
      {
      win = WindowFind("Bell_Curve_demo");
      }

   if(!bellDone) Bell();
   if(!gridDone) GridP();

   return(0);
   } // end int start()

//+------------------------------------------------------------------+

void Bell()
   {
   // bell lines & SD markers
   TrendLine("base", win, base + 330, base -330 , 0, 0, clrSig, 0, 0, false, false);
   TrendLine("mean", win, base, base , 0, 0.4, clrSig, STYLE_DOT, 1, false, false);

   for(int i=-3; i<=3; i++)
      {
      string obj = StringConcatenate("SD", i);
      TrendLine(obj, win, zBar(i), zBar(i), 0, 0.03,  clrSig, 0, 2, false, true);
      }

   for(i=-3; i<=3; i++)
      {
      obj = StringConcatenate("Sig", i);
      string text = StringConcatenate(GetSign(i), i);
      TextBox(obj, win, zBar(i), -0.005,  text, fontSize + 1, font, clrSig);
      }

   // Draw the curve using simple and approximated pdf function: Var = 1, SD = 1, Mu = 0
   // Z-score is incremented by step 0.01. Resulting in a graduation of 100 sub-units per SD unit
   // Curve is drawn for Z = -PI to Z = +PI (3.14)

   double z = 0;
   for(i=base; i>=base -314 ;i--)
      {
      bell[i] = 0.4 * (MathPow(2.72, -MathPow(z, 2) / 2));
      z += 0.01;
      }
   z = 0;
   for(i=base; i<=base +314 ;i++)
      {
      bell[i] = 0.4 * (MathPow(2.72, -MathPow(z, 2) / 2));
      z -= 0.01;
      }

   bellDone = true;
   return;
   } // end void Bell()

//+------------------------------------------------------------------+

void GridP()
   {
   TrendLine("gridbase", win, base + 330, base -330 , 0.50, 0.50, clrSig, 2, 0, false, false);
   for(int i= 0; i<= 11; i++)
      { // set various hashmarks and display critical values
      int tag = zBar(critZp[i]);

      string obj = StringConcatenate("dashP_", i);
      TrendLine(obj, win, tag, tag, 0.51, 0.49,  clrSig, 0, 0, false, true);

      obj = StringConcatenate("hashP_", i);
      TrendLine(obj, win, tag, tag, GetY(critZp[i])+0.01, GetY(critZp[i])-0.01,  clrSig, 0, 0, false, true);

      obj = StringConcatenate("critZp_", i);
      string text = StringConcatenate(GetSign(critZp[i]), DoubleToStr(critZp[i],2));
      TextBox(obj, win, tag, 0.55, text, fontSize-1, font, clrSig);

      obj = StringConcatenate("critP_", i);
      TextBox(obj, win, tag, 0.48, DoubleToStr(critP[i],3), fontSize-1, font, clrSig);

      obj = StringConcatenate("critP%_", i);
      text = StringConcatenate(DoubleToStr(critP[i]*100,1), "%");
      TextBox(obj, win, tag, 0.59, text, fontSize-1, font, clrSig);
      }

   for(i= 0; i<= 6; i++)
      {
      tag = zBar(percZp[i]);
      obj = StringConcatenate("percP_", i);
      TrendLine(obj, win, tag, tag, 0.51, 0.49,  clrSig, 0, 0, false, true);
      }
   double box = 0.80;
   for(i=0; i<=3; i++)
      {
      tag = zBar(iqR[i]);
      obj = StringConcatenate("iqrfix_", i);
      TrendLine(obj, win, tag , tag, box + 0.02, box - 0.02, clrSig, 0, 0, false, true);

      obj = StringConcatenate("iqrtxt_", i);
      text = StringConcatenate(GetSign(iqR[i]), DoubleToStr(iqR[i], 4));
      TextBox(obj, win, tag, box - 0.06, text, fontSize-1, font, clrSig);
      }
   gridDone = true;
   return;;
   } // end void GridP()

//+---------------------------FUNCTIONS------------------------------+

double GetY(double z)
   { // get y value from a known Z-score value
   double y = 0.4 * MathPow(2.72, -(MathPow(z, 2) / 2));

   return(y);
   } // end double GetY()

//+------------------------------------------------------------------+

string GetSign(double in)
   {
   if (in <= 0) {string sign = "";} else {sign = "+";}
   return(sign);
   } // end string GetSign(double in)

//+------------------------------------------------------------------+

int zBar(double z)
   {// find bar number to display a known Z-score
   int zbar = base  - MathRound(z*100);
   return(zbar);
   } // end int zBar()

//+---------------------GRAPHICS-------------------------------------+

void TrendLine(string obj, int winIdx, int t1, int t2, double p1, double p2, color clr, int st, int w, bool ray, bool back)
   {
   obj = StringConcatenate(pref, obj);
   if(!ObjectCreate(obj, OBJ_TREND,winIdx,0,0,0,0)) ObjectCreate(obj, OBJ_TREND,winIdx,0,0,0,0);
      if(!ObjectSet(obj, OBJPROP_TIME1, Time[t1]))  ObjectSet(obj, OBJPROP_TIME1, Time[t1]);
      if(!ObjectSet(obj, OBJPROP_TIME2, Time[t2]))  ObjectSet(obj, OBJPROP_TIME2, Time[t2]);
      if(!ObjectSet(obj, OBJPROP_PRICE1, p1))       ObjectSet(obj, OBJPROP_PRICE1, p1);
      if(!ObjectSet(obj, OBJPROP_PRICE2, p2))       ObjectSet(obj, OBJPROP_PRICE2, p2);
      if(!ObjectSet(obj, OBJPROP_COLOR,  clr))      ObjectSet(obj, OBJPROP_COLOR,  clr);
      if(!ObjectSet(obj, OBJPROP_STYLE,  st))       ObjectSet(obj, OBJPROP_STYLE,  st);
      if(!ObjectSet(obj, OBJPROP_WIDTH,  w))        ObjectSet(obj, OBJPROP_WIDTH,  w);
      if(!ObjectSet(obj, OBJPROP_RAY,    ray))      ObjectSet(obj, OBJPROP_RAY,    ray);
      if(!ObjectSet(obj, OBJPROP_BACK,   back))     ObjectSet(obj, OBJPROP_BACK,   back);

   return;
   }// end void TrendLine(

//+------------------------------------------------------------------+

void TextBox(string obj, int winIdx, int t1, double p1, string text, int size, string afont, color clr)
   {
   obj = StringConcatenate(pref, obj);
   if(!ObjectCreate(obj, OBJ_TEXT, winIdx,0,0,0)) ObjectCreate(obj, OBJ_TEXT, winIdx,0,0,0);
      if(!ObjectSet(obj, OBJPROP_TIME1, Time[t1]))   ObjectSet(obj, OBJPROP_TIME1, Time[t1]);
      if(!ObjectSet(obj, OBJPROP_PRICE1, p1))        ObjectSet(obj, OBJPROP_PRICE1, p1);
      if(!ObjectSet(obj, OBJPROP_ANGLE, 0))          ObjectSet(obj, OBJPROP_ANGLE, 0);
   if(!ObjectSetText(obj, text , size, afont, clr)) ObjectSetText(obj, text , size, afont, clr);

   return;
   }// end void TextBox(

//+---------------------------END FUNCTIONS--------------------------+

//+--------------------------------EOP-------------------------------+

